@@ -24,7 +24,7 @@
 #include "gtr-application.h"
 #include "gtr-window.h"
 #include "gtr-header.h"
-#include "gtr-plugin.h"
+#include "gtr-dirs.h"
 
 #include <string.h>
 
@@ -46,18 +46,18 @@
 						  GTR_TYPE_OPEN_TRAN_PANEL, \
 						  GtrOpenTranPanelPrivate))
 
-#define GNOME_ICON       PIXMAPSDIR"/gnome.png"
-#define KDE_ICON         PIXMAPSDIR"/kde.ico"
-#define MOZILLA_ICON     PIXMAPSDIR"/mozilla.png"
-#define DEBIAN_ICON      PIXMAPSDIR"/debian.png"
-#define SUSE_ICON        PIXMAPSDIR"/suse.png"
-#define XFCE_ICON        PIXMAPSDIR"/xfce.png"
-#define INKSCAPE_ICON    PIXMAPSDIR"/inkscape.png"
-#define OPEN_OFFICE_ICON PIXMAPSDIR"/oo-logo.png"
-#define FEDORA_ICON      PIXMAPSDIR"/fedora.png"
-#define MANDRIVA_ICON    PIXMAPSDIR"/mandriva.png"
+#define GNOME_ICON       "gnome.png"
+#define KDE_ICON         "kde.ico"
+#define MOZILLA_ICON     "mozilla.png"
+#define DEBIAN_ICON      "debian.png"
+#define SUSE_ICON        "suse.png"
+#define XFCE_ICON        "xfce.png"
+#define INKSCAPE_ICON    "inkscape.png"
+#define OPEN_OFFICE_ICON "oo-logo.png"
+#define FEDORA_ICON      "fedora.png"
+#define MANDRIVA_ICON    "mandriva.png"
 
-GTR_PLUGIN_DEFINE_TYPE (GtrOpenTranPanel, gtr_open_tran_panel, GTK_TYPE_VBOX)
+G_DEFINE_DYNAMIC_TYPE (GtrOpenTranPanel, gtr_open_tran_panel, GTK_TYPE_VBOX)
 
 struct _GtrOpenTranPanelPrivate
 {
@@ -106,7 +106,7 @@ show_error_dialog (GtrWindow * parent,
 
 static gchar *
 get_service_url (gboolean use_mirror_server, gchar *mirror_server_url,
-		 const gchar * search_text,
+                 const gchar * search_text,
                  const gchar * search_code, const gchar * own_code)
 {
   const gchar *base_url;
@@ -131,12 +131,17 @@ get_service_url (gboolean use_mirror_server, gchar *mirror_server_url,
 }
 
 static GdkPixbuf *
-create_pixbuf (const gchar * path)
+create_pixbuf (const gchar *filename)
 {
   GdkPixbuf *icon;
+  gchar *path;
   GError *error = NULL;
 
+  path = g_build_filename (gtr_dirs_get_gtr_pixmaps_dir (),
+                           filename,
+                           NULL);
   icon = gdk_pixbuf_new_from_file (path, &error);
+  g_free (path);
 
   if (error)
     {
@@ -538,7 +543,7 @@ gtr_open_tran_panel_dispose (GObject * object)
 }
 
 static void
-gtr_open_tran_panel_class_init (GtrOpenTranPanelClass * klass)
+gtr_open_tran_panel_class_init (GtrOpenTranPanelClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
@@ -547,6 +552,11 @@ gtr_open_tran_panel_class_init (GtrOpenTranPanelClass * klass)
   object_class->dispose = gtr_open_tran_panel_dispose;
 }
 
+static void
+gtr_open_tran_panel_class_finalize (GtrOpenTranPanelClass *klass)
+{
+}
+
 GtkWidget *
 gtr_open_tran_panel_new (GtrWindow * window)
 {
@@ -558,3 +568,9 @@ gtr_open_tran_panel_new (GtrWindow * window)
 
   return GTK_WIDGET (panel);
 }
+
+void
+_gtr_open_tran_panel_register_type (GTypeModule *type_module)
+{
+  gtr_open_tran_panel_register_type (type_module);
+}
