@@ -26,6 +26,7 @@
 
 #include "gtr-actions.h"
 #include "gtr-application.h"
+#include "gtr-debug.h"
 #include "gtr-dirs.h"
 #include "gtr-header.h"
 #include "gtr-msg.h"
@@ -37,6 +38,7 @@
 #include "gtr-statusbar.h"
 #include "gtr-utils.h"
 #include "gtr-window.h"
+#include "gtr-window-activatable.h"
 #include "gtr-profile-manager.h"
 #include "gtr-status-combo-box.h"
 
@@ -55,6 +57,7 @@
 #include <glib-object.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <libpeas/peas-extension-set.h>
 
 #define GTR_STOCK_FUZZY_NEXT "gtranslator-fuzzy-next"
 #define GTR_STOCK_FUZZY_PREV "gtranslator-fuzzy-prev"
@@ -113,7 +116,8 @@ struct _GtrWindowPrivate
   GtrProfileManager *prof_manager;
   GtkWidget *profile_combo;
 
-  guint destroy_has_run : 1;
+  PeasExtensionSet *extensions;
+
   guint dispose_has_run : 1;
 };
 
@@ -359,12 +363,11 @@ gtr_window_layout_load (GtrWindow * window,
       !gdl_dock_layout_load_from_file (window->priv->layout_manager,
                                        layout_filename))
     {
+      const gchar *datadir;
       gchar *path;
-      gchar *datadir;
 
       datadir = gtr_dirs_get_gtr_data_dir ();
       path = g_build_filename (datadir, "layout.xml", NULL);
-      g_free (datadir);
 
       //DEBUG_PRINT ("Layout = %s", path);
       if (!gdl_dock_layout_load_from_file (window->priv->layout_manager,
@@ -545,7 +548,8 @@ remove_widget (GtrWindow * window, GtkWidget * widget, GError ** error)
 }
 
 void
-set_sensitive_according_to_message (GtrWindow * window, GtrPo * po)
+_gtr_window_set_sensitive_according_to_message (GtrWindow * window,
+                                                GtrPo * po)
 {
   GList *current;
   GtkAction *action;
@@ -602,7 +606,6 @@ set_sensitive_according_to_tab (GtrWindow * window, GtrTab * tab)
   GtrPo *po;
   GtkSourceBuffer *buf;
   GtkAction *action;
-  GList *current;
   GtrPoState state;
   gint pages;
   gint current_page;
@@ -611,7 +614,6 @@ set_sensitive_according_to_tab (GtrWindow * window, GtrTab * tab)
   pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->priv->notebook));
   view = gtr_tab_get_active_view (tab);
   po = gtr_tab_get_po (tab);
-  current = gtr_po_get_current_message (po);
   buf = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
   if (gtk_action_group_get_sensitive (window->priv->action_group) == FALSE)
@@ -657,10 +659,12 @@ set_sensitive_according_to_tab (GtrWindow * window, GtrTab * tab)
                                         "DocumentsNextDocument");
   gtk_action_set_sensitive (action, current_page < pages - 1);
 
-  set_sensitive_according_to_message (window, po);
+  _gtr_window_set_sensitive_according_to_message (window, po);
+
+  peas_extension_set_call (window->priv->extensions, "update_state", window);
 }
 
-void
+static void
 set_sensitive_according_to_window (GtrWindow * window)
 {
   gint pages;
@@ -912,8 +916,7 @@ drag_data_received_cb (GtkWidget * widget,
       locations = gtr_utils_drop_get_locations (selection_data);
       gtr_actions_load_locations (window, locations);
 
-      g_slist_foreach (locations, (GFunc) g_object_unref, NULL);
-      g_slist_free (locations);
+      g_slist_free_full (locations, g_object_unref);
     }
 }
 
@@ -1056,8 +1059,7 @@ notebook_switch_page (GtkNotebook * nb,
   if (action != NULL)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 
-  gtr_plugins_engine_update_plugins_ui
-    (gtr_plugins_engine_get_default (), window, FALSE);
+  peas_extension_set_call (window->priv->extensions, "update_state", window);
 }
 
 static void
@@ -1078,6 +1080,8 @@ notebook_page_removed (GtkNotebook * notebook,
     gtk_widget_hide (window->priv->profile_combo);
 
   update_documents_list_menu (window);
+
+  peas_engine_garbage_collect (PEAS_ENGINE (gtr_plugins_engine_get_default ()));
 }
 
 static void
@@ -1088,8 +1092,7 @@ notebook_tab_close_request (GtrNotebook * notebook,
    * seems to be ok, but we need to keep an eye on this. */
   gtr_close_tab (tab, window);
 
-  gtr_plugins_engine_update_plugins_ui
-    (gtr_plugins_engine_get_default (), window, FALSE);
+  peas_extension_set_call (window->priv->extensions, "update_state", window);
 }
 
 static void
@@ -1153,7 +1156,7 @@ showed_message_cb (GtrTab * tab, GtrMsg * msg, GtrWindow * window)
 
   gtr_window_update_statusbar_message_count (tab, msg, window);
 
-  set_sensitive_according_to_message (window, gtr_tab_get_po (tab));
+  _gtr_window_set_sensitive_according_to_message (window, gtr_tab_get_po (tab));
 }
 
 static void
@@ -1210,8 +1213,7 @@ notebook_tab_added (GtkNotebook * notebook,
 
   update_documents_list_menu (window);
 
-  gtr_plugins_engine_update_plugins_ui (gtr_plugins_engine_get_default (),
-                                        window, FALSE);
+  peas_extension_set_call (window->priv->extensions, "update_state", window);
 }
 
 void
@@ -1293,8 +1295,7 @@ gtr_recent_chooser_item_activated_cb (GtkRecentChooser * chooser,
   list = g_slist_prepend (list, location);
 
   gtr_actions_load_locations (window, list);
-  g_slist_foreach (list, (GFunc) g_object_unref, NULL);
-  g_slist_free (list);
+  g_slist_free_full (list, g_object_unref);
 }
 
 static GtkWidget *
@@ -1347,7 +1348,6 @@ gtr_window_cmd_edit_toolbar (GtkAction * action, GtrWindow * window)
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
   gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)), 5);
   gtk_box_set_spacing (GTK_BOX (content_area), 2);
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   gtk_window_set_default_size (GTK_WINDOW (dialog), 500, 400);
 
   editor = egg_toolbar_editor_new (window->priv->ui_manager,
@@ -1357,7 +1357,7 @@ gtr_window_cmd_edit_toolbar (GtkAction * action, GtrWindow * window)
   gtk_container_set_border_width (GTK_CONTAINER (editor), 5);
   gtk_box_set_spacing (GTK_BOX (EGG_TOOLBAR_EDITOR (editor)), 5);
 
-  gtk_container_add (GTK_CONTAINER (content_area), editor);
+  gtk_box_pack_start (GTK_BOX (content_area), editor, TRUE, TRUE, 0);
 
   egg_editable_toolbar_set_edit_mode
     (EGG_EDITABLE_TOOLBAR (window->priv->toolbar), TRUE);
@@ -1629,7 +1629,7 @@ gtr_window_draw (GtrWindow * window)
   GError *error = NULL;
   GtkWidget *dockbar;
   GtkActionGroup *action_group;
-  gchar *datadir;
+  const gchar *datadir;
   gchar *path;
 
   GtrWindowPrivate *priv = window->priv;
@@ -1675,7 +1675,6 @@ gtr_window_draw (GtrWindow * window)
 
   datadir = gtr_dirs_get_gtr_data_dir ();
   path = g_build_filename (datadir, "gtranslator-ui.xml", NULL);
-  g_free (datadir);
 
   if (!gtk_ui_manager_add_ui_from_file (priv->ui_manager, path, &error))
     {
@@ -1721,6 +1720,9 @@ gtr_window_draw (GtrWindow * window)
                    "model",
                    _gtr_application_get_toolbars_model (GTR_APP), NULL));
 
+  gtk_style_context_add_class (gtk_widget_get_style_context (priv->toolbar),
+                               GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
+
   egg_editable_toolbar_show (EGG_EDITABLE_TOOLBAR (priv->toolbar),
                              "DefaultToolBar");
   gtk_box_pack_start (GTK_BOX (priv->main_box),
@@ -1770,20 +1772,43 @@ gtr_window_draw (GtrWindow * window)
   create_statusbar (window, hbox);
 }
 
+static void
+extension_added (PeasExtensionSet *extensions,
+                 PeasPluginInfo   *info,
+                 PeasExtension    *exten,
+                 GtrWindow        *window)
+{
+  peas_extension_call (exten, "activate");
+}
+
+static void
+extension_removed (PeasExtensionSet *extensions,
+                   PeasPluginInfo   *info,
+                   PeasExtension    *exten,
+                   GtrWindow        *window)
+{
+  peas_extension_call (exten, "deactivate");
+
+  /* Ensure update of ui manager, because we suspect it does something
+   * with expected static strings in the type module (when unloaded the
+   * strings don't exist anymore, and ui manager updates in an idle
+   * func) */
+  gtk_ui_manager_ensure_update (window->priv->ui_manager);
+}
+
 static void
 gtr_window_init (GtrWindow * window)
 {
   GtkTargetList *tl;
   GtkWidget *view_menu;
+  const gchar *config_folder;
   gchar *filename;
-  gchar *config_folder;
 
   window->priv = GTR_WINDOW_GET_PRIVATE (window);
 
   window->priv->ui_settings = g_settings_new ("org.gnome.gtranslator.preferences.ui");
   window->priv->state_settings = g_settings_new ("org.gnome.gtranslator.state.window");
 
-  window->priv->destroy_has_run = FALSE;
   window->priv->dispose_has_run = FALSE;
 
   /* Profile manager */
@@ -1834,10 +1859,6 @@ gtr_window_init (GtrWindow * window)
   window->priv->view_menu =
     gtk_menu_item_get_submenu (GTK_MENU_ITEM (view_menu));
 
-  /* Plugins */
-  gtr_plugins_engine_update_plugins_ui
-    (gtr_plugins_engine_get_default (), window, TRUE);
-
   /* Adding notebook to dock */
   add_widget_full (window,
                    window->priv->notebook,
@@ -1848,17 +1869,31 @@ gtr_window_init (GtrWindow * window)
   /* Loading dock layout */
   config_folder = gtr_dirs_get_user_config_dir ();
   filename = g_build_filename (config_folder, "gtr-layout.xml", NULL);
-  g_free (config_folder);
 
   gtr_window_layout_load (window, filename, NULL);
   g_free (filename);
+
+  /* Plugins */
+  window->priv->extensions = peas_extension_set_new (PEAS_ENGINE (gtr_plugins_engine_get_default ()),
+                                                     GTR_TYPE_WINDOW_ACTIVATABLE,
+                                                     "window", window,
+                                                     NULL);
+  g_signal_connect (window->priv->extensions,
+                    "extension-added",
+                    G_CALLBACK (extension_added),
+                    window);
+  g_signal_connect (window->priv->extensions,
+                    "extension-removed",
+                    G_CALLBACK (extension_removed),
+                    window);
+  peas_extension_set_call (window->priv->extensions, "activate");
 }
 
 static void
 save_panes_state (GtrWindow * window)
 {
+  const gchar *config_folder;
   gchar *filename;
-  gchar *config_folder;
 
   g_settings_set (window->priv->state_settings, GTR_SETTINGS_WINDOW_SIZE, "(ii)",
                   window->priv->width, window->priv->height);
@@ -1867,7 +1902,6 @@ save_panes_state (GtrWindow * window)
 
   config_folder = gtr_dirs_get_user_config_dir ();
   filename = g_build_filename (config_folder, "gtr-layout.xml", NULL);
-  g_free (config_folder);
 
   gtr_window_layout_save (window, filename, NULL);
   g_free (filename);
@@ -1881,9 +1915,21 @@ gtr_window_dispose (GObject * object)
 
   DEBUG_PRINT ("window dispose");
 
+  /* First of all, force collection so that plugins
+   * really drop some of the references.
+   */
+  peas_engine_garbage_collect (PEAS_ENGINE (gtr_plugins_engine_get_default ()));
+
   if (!priv->dispose_has_run)
     {
       save_panes_state (window);
+
+      /* Note that unreffing the extensions will automatically remove
+         all extensions which in turn will deactivate the extension */
+      g_object_unref (priv->extensions);
+
+      peas_engine_garbage_collect (PEAS_ENGINE (gtr_plugins_engine_get_default ()));
+
       priv->dispose_has_run = TRUE;
     }
 
@@ -1916,10 +1962,22 @@ gtr_window_dispose (GObject * object)
       priv->prof_manager = NULL;
     }
 
+  if (priv->widgets)
+    {
+      g_hash_table_unref (priv->widgets);
+      priv->widgets = NULL;
+    }
+
+  if (priv->layout_manager)
+    {
+      g_object_unref (priv->layout_manager);
+      priv->layout_manager = NULL;
+    }
+
   /* Now that there have broken some reference loops,
    * force collection again.
    */
-  gtr_plugins_engine_garbage_collect (gtr_plugins_engine_get_default ());
+  peas_engine_garbage_collect (PEAS_ENGINE (gtr_plugins_engine_get_default ()));
 
   G_OBJECT_CLASS (gtr_window_parent_class)->dispose (object);
 }
@@ -1927,38 +1985,11 @@ gtr_window_dispose (GObject * object)
 static void
 gtr_window_finalize (GObject * object)
 {
+  DEBUG_PRINT ("Finalize window");
+
   G_OBJECT_CLASS (gtr_window_parent_class)->finalize (object);
 }
 
-static void
-gtr_window_destroy (GtkObject * object)
-{
-  GtrWindow *window;
-
-  window = GTR_WINDOW (object);
-
-  DEBUG_PRINT ("Destroy window");
-
-  if (!window->priv->destroy_has_run)
-    {
-      if (window->priv->widgets)
-        {
-          g_hash_table_destroy (window->priv->widgets);
-          window->priv->widgets = NULL;
-        }
-
-      if (window->priv->layout_manager)
-        {
-          g_object_unref (window->priv->layout_manager);
-          window->priv->layout_manager = NULL;
-        }
-      window->priv->destroy_has_run = TRUE;
-    }
-
-  GTK_OBJECT_CLASS (gtr_window_parent_class)->destroy (object);
-}
-
-
 static gboolean
 gtr_window_configure_event (GtkWidget * widget, GdkEventConfigure * event)
 {
@@ -1971,6 +2002,8 @@ gtr_window_configure_event (GtkWidget * widget, GdkEventConfigure * event)
                                                                       event);
 }
 
+#if 0
+FIXME: for now we are giving priority to the window accels
 /*
  * GtkWindow catches keybindings for the menu items _before_ passing them to
  * the focused widget. This is unfortunate and means that pressing ctrl+V
@@ -2004,12 +2037,12 @@ gtr_window_key_press_event (GtkWidget   *widget,
 
   return handled;
 }
+#endif
 
 static void
 gtr_window_class_init (GtrWindowClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GtkObjectClass *gobject_class = GTK_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (GtrWindowPrivate));
@@ -2017,10 +2050,8 @@ gtr_window_class_init (GtrWindowClass * klass)
   object_class->finalize = gtr_window_finalize;
   object_class->dispose = gtr_window_dispose;
 
-  gobject_class->destroy = gtr_window_destroy;
-
   widget_class->configure_event = gtr_window_configure_event;
-  widget_class->key_press_event = gtr_window_key_press_event;
+  /*widget_class->key_press_event = gtr_window_key_press_event;*/
 }
 
 /***************************** Public funcs ***********************************/
@@ -2033,7 +2064,7 @@ gtr_window_class_init (GtrWindowClass * klass)
  * Adds a new #GtrTab to the #GtrNotebook and returns the
  * #GtrTab.
  * 
- * Returns: a new #GtrTab object
+ * Returns: (transfer none): a new #GtrTab object
  */
 GtrTab *
 gtr_window_create_tab (GtrWindow * window, GtrPo * po)
@@ -2054,7 +2085,7 @@ gtr_window_create_tab (GtrWindow * window, GtrPo * po)
  * 
  * Gets the active #GtrTab of the @window.
  *
- * Returns: the active #GtrTab of the @window.
+ * Returns: (transfer none): the active #GtrTab of the @window.
  */
 GtrTab *
 gtr_window_get_active_tab (GtrWindow * window)
@@ -2068,7 +2099,8 @@ gtr_window_get_active_tab (GtrWindow * window)
  *
  * Gets a list of all tabs in the @window or NULL if there is no tab opened.
  *
- * Returns: a list of all tabs in the @window or NULL if there is no tab opened.
+ * Returns: (transfer container): a list of all tabs in the @window or NULL if there
+ * is no tab opened.
  */
 GList *
 gtr_window_get_all_tabs (GtrWindow * window)
@@ -2099,7 +2131,7 @@ gtr_window_get_all_tabs (GtrWindow * window)
  * Gets the #GtrHeader of the #GtrPo of in the active
  * #GtrTab.
  *
- * Returns: the #GtrHeader of the #GtrPo of in the active
+ * Returns: (transfer none): the #GtrHeader of the #GtrPo of in the active
  * #GtrTab
  */
 GtrHeader *
@@ -2127,7 +2159,7 @@ gtr_window_get_header_from_active_tab (GtrWindow * window)
  * 
  * Gets the main #GtrNotebook of the @window.
  *
- * Returns: the #GtrNotebook of the @window
+ * Returns: (transfer none): the #GtrNotebook of the @window
  */
 GtrNotebook *
 gtr_window_get_notebook (GtrWindow * window)
@@ -2141,7 +2173,7 @@ gtr_window_get_notebook (GtrWindow * window)
  *
  * Gets the statusbar widget of the window.
  *
- * Returns: the statusbar widget of the window
+ * Returns: (transfer none): the statusbar widget of the window
  */
 GtkWidget *
 gtr_window_get_statusbar (GtrWindow * window)
@@ -2155,7 +2187,7 @@ gtr_window_get_statusbar (GtrWindow * window)
  *
  * Gets the #GtkUIManager of the window.
  *
- * Returns: the #GtkUIManager of the @window
+ * Returns: (transfer none): the #GtkUIManager of the @window
  */
 GtkUIManager *
 gtr_window_get_ui_manager (GtrWindow * window)
@@ -2170,8 +2202,8 @@ gtr_window_get_ui_manager (GtrWindow * window)
  * Gets the active translation view in the #GtranslationWindow or
  * NULL if there is not tab opened.
  *
- * Returns: the active translation view in the #GtranslationWindow or
- * NULL if there is not tab opened.
+ * Returns: (transfer none): the active translation view in the
+ *          #GtranslationWindow or %NULL if there is not tab opened.
  **/
 GtrView *
 gtr_window_get_active_view (GtrWindow * window)
@@ -2193,7 +2225,7 @@ gtr_window_get_active_view (GtrWindow * window)
  *
  * Returns all the views currently present in #GtranslationWindow
  *
- * Return value: a newly allocated list of #GtranslationWindow objects
+ * Return value: (transfer container): a newly allocated list of #GtranslationWindow objects
  **/
 GList *
 gtr_window_get_all_views (GtrWindow * window,
@@ -2278,14 +2310,14 @@ _gtr_window_get_layout_manager (GtrWindow * window)
 }
 
 /**
- * gtr_window_get_tab_from_uri:
+ * gtr_window_get_tab_from_location:
  * @window: a #GtrWindow
  * @location: the GFile of the po file of the #GtrTab
  *
  * Gets the #GtrTab of the #GtrWindows that matches with the
  * @location.
  *
- * Returns: the #GtrTab which @location matches with its po file.
+ * Returns: (transfer none): the #GtrTab which @location matches with its po file.
  */
 GtkWidget *
 gtr_window_get_tab_from_location (GtrWindow * window, GFile * location)
@@ -2365,6 +2397,12 @@ _gtr_window_close_tab (GtrWindow * window, GtrTab * tab)
   set_sensitive_according_to_window (window);
 }
 
+/**
+ * gtr_window_get_tm_menu:
+ * @window: a #GtrWindow
+ *
+ * Returns: (transfer none): get the tm_menu GtkWidget instance.
+ */
 GtkWidget *
 gtr_window_get_tm_menu (GtrWindow * window)
 {
